@@ -5,7 +5,9 @@
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
 #
 """This module contains functions for parsing ``funpack`` command line
-arguments and configuration files.
+arguments and configuration files. A ``funpack`` configuration file
+is simply a plain-text file which contains command-line options,
+sans-hyphens.
 """
 
 
@@ -54,6 +56,7 @@ CLI_ARGUMENTS              = collections.OrderedDict((
                                         'default' : DEFAULT_MERGE_AXIS}),
         (('ms',  'merge_strategy'),    {'choices' : AVAILABLE_MERGE_STRATEGIES,
                                         'default' : DEFAULT_MERGE_STRATEGY}),
+        (('rm',  'remove_duplicates'), {'action' : 'store_true'}),
         (('rd',  'rename_duplicates'), {'action' : 'store_true'}),
         (('cfg', 'config_file'),       {'action'  : 'append'}),
         (('vf',  'variable_file'),     {'action'  : 'append'}),
@@ -63,14 +66,16 @@ CLI_ARGUMENTS              = collections.OrderedDict((
         (('cf',  'category_file'),     {})]),
 
     ('Import options', [
-        (('s',  'subject'),        {'action' : 'append'}),
-        (('v',  'variable'),       {'action' : 'append'}),
-        (('co', 'column'),         {'action' : 'append'}),
-        (('c',  'category'),       {'action' : 'append'}),
-        (('vi', 'visit'),          {'action' : 'append'}),
-        (('ex', 'exclude'),        {'action' : 'append'}),
-        (('iv', 'index_visits'),   {'action' : 'store_true'}),
-        (('tt', 'trust_types'),    {'action' : 'store_true'})]),
+        (('s',  'subject'),          {'action' : 'append'}),
+        (('v',  'variable'),         {'action' : 'append'}),
+        (('co', 'column'),           {'action' : 'append'}),
+        (('c',  'category'),         {'action' : 'append'}),
+        (('vi', 'visit'),            {'action' : 'append'}),
+        (('ex', 'exclude'),          {'action' : 'append'}),
+        (('ev', 'exclude_variable'), {'action' : 'append'}),
+        (('ec', 'exclude_category'), {'action' : 'append'}),
+        (('iv', 'index_visits'),     {'action' : 'store_true'}),
+        (('tt', 'trust_types'),      {'action' : 'store_true'})]),
 
     ('Cleaning options', [
         (('sn',  'skip_insertna'),      {'action'  : 'store_true'}),
@@ -224,6 +229,9 @@ CLI_ARGUMENT_HELP = {
     'Options are "naive", "intersection"/"inner", or "union"/'
     '"outer".'.format(DEFAULT_MERGE_STRATEGY),
 
+    'remove_duplicates' :
+    'Remove duplicate columns, only retaining the first.',
+
     'rename_duplicates' :
     'Rename any duplicate columns so that all columns have a unique name.',
 
@@ -277,6 +285,15 @@ CLI_ARGUMENT_HELP = {
     'Subject ID, range, comma-separated list, or file containing a list of '
     'subject IDs specifying subjects to exclude. Can be used multiple times.',
 
+    'exclude_variable' :
+    'Variable ID, range, comma-separated list, or file containing a list of '
+    'variable IDs, to exclude. Takes precedence over the --variable and '
+    '--category options. Can be used multiple times.',
+
+    'exclude_category' :
+    'Category ID or label to exclude. Takes precedence over the --variable '
+    'and --category options. Can be used multiple times.',
+
     'index_visits' :
     'If set, the data is re-arranged so that visits form part of the row '
     'indices, rather than being stored in separate columns for each variable. '
@@ -686,6 +703,11 @@ def _prepareInputAndOutputFiles(args):
     if args.loader is not None:
         args.loader = {op.realpath(f) : n for f, n in args.loader}
 
+    # Remove/rename duplicates options are mutually exclusive
+    if args.remove_duplicates and args.rename_duplicates:
+        raise ValueError('Only one of--remove_duplicates and '
+                         '--rename_duplicates may be used.')
+
     # turn index indices into dict of
     # { file : [index] } mappings
     if args.index is not None:
@@ -791,10 +813,11 @@ def _prepareSubjectAndVariableSelectors(args):
     :arg args: ``argparse.Namespace`` object.
     """
 
-    # turn --subject/--variable/--exclude
-    # arguments into lists of IDs. If
-    # error is True, an error is raised on
-    # unparseable arguments.
+    # turn --subject/--variable/--exclude/
+    # --exclude_variable arguments into
+    # lists of IDs. If error is True, an
+    # error is raised on unparseable
+    # arguments.
     def replaceIDs(things, error=True):
         newthings = []
         failed    = []
@@ -845,11 +868,17 @@ def _prepareSubjectAndVariableSelectors(args):
     # a list of integer IDs, but subject is
     # transformed into a tuple containing
     # ([ID], [exprStr])
-    if args.subject  is not None: args.subject  = replaceIDs(args.subject,
-                                                             False)
-    else:                         args.subject  = None, None
-    if args.variable is not None: args.variable = replaceIDs(args.variable)[0]
-    if args.exclude  is not None: args.exclude  = replaceIDs(args.exclude)[0]
+    if args.subject is not None:
+        args.subject  = replaceIDs(args.subject, False)
+    else:
+        args.subject  = None, None
+
+    if args.variable is not None:
+        args.variable = replaceIDs(args.variable)[0]
+    if args.exclude is not None:
+        args.exclude = replaceIDs(args.exclude)[0]
+    if args.exclude_variable is not None:
+        args.exclude_variable  = replaceIDs(args.exclude_variable)[0]
 
 
 def _prepareCategorySelectors(args):
@@ -866,6 +895,10 @@ def _prepareCategorySelectors(args):
         for i, c in enumerate(args.category):
             try:               args.category[i] = int(c)
             except ValueError: continue
+    if args.exclude_category is not None:
+        for i, c in enumerate(args.exclude_category):
+            try:               args.exclude_category[i] = int(c)
+            except ValueError: continue
 
 
 def _prepareColumnSelectors(args):
