@@ -298,7 +298,8 @@ class FileInfo:
                  indexes=None,
                  loaders=None,
                  encodings=None,
-                 renameDuplicates=False):
+                 renameDuplicates=False,
+                 renameSuffix=None):
         """Create a ``FileInfo`` object.
 
         :arg datafiles:        Path to input file, or sequence of paths.
@@ -311,6 +312,9 @@ class FileInfo:
                                specifying non-standard file encodings.
         :arg renameDuplicates: If ``True``, duplicate columns are re-named -
                                see :func:`renameDuplicateColumns`.
+        :arg renameSuffix:     Passed as ``suffix`` to
+                               :func:`renameDuplicateColumns`, if
+                               ``renameDuplicates is True``.
         """
 
         if isinstance(datafiles, str): datafiles = [datafiles]
@@ -322,7 +326,8 @@ class FileInfo:
                                            indexes,
                                            loaders,
                                            encodings,
-                                           renameDuplicates)
+                                           renameDuplicates,
+                                           renameSuffix=renameSuffix)
 
         self.__datafiles = list(datafiles)
         self.__indexes   = dict(indexes)
@@ -388,7 +393,8 @@ def fileinfo(datafiles,
              indexes=None,
              sniffers=None,
              encodings=None,
-             renameDuplicates=False):
+             renameDuplicates=False,
+             renameSuffix=None):
     """Identifies the format of each input data file, and extracts/generates
     column names and variable IDs for every column.
 
@@ -410,6 +416,10 @@ def fileinfo(datafiles,
                            which have the same name are renamed - see
                            :func:`renameDuplicateColumns`.
 
+    :arg renameSuffix:     Passed as ``suffix`` to
+                           :func:`renameDuplicateColumns`, if
+                           ``renameDuplicates is True``.
+
     :returns: A tuple containing:
 
                - List of ``csv`` dialect types
@@ -517,22 +527,28 @@ def fileinfo(datafiles,
                 col.name = util.generateColumnName(vid, 0, 0)
 
     if renameDuplicates:
-        renameDuplicateColumns(it.chain(*cols))
+        renameDuplicateColumns(it.chain(*cols), suffix=renameSuffix)
 
     return dialects, headers, cols
 
 
-def renameDuplicateColumns(cols):
+def renameDuplicateColumns(cols, suffix=None):
     """Identifies any columns which have the same name, and re-names the
     subsequent ones.  If ``N`` columns have the same name ``X``, they are
-    renamed ``X``, ``X.1``, ``X.2``, ``...``, ``X.<N-1>``.
+    renamed ``X``, ``X.1<suffix>``, ``X.2<suffix>``, ``...``,
+    ``X.<N-1><suffix>``.
 
     The ``name`` attribute of each :class:`.Column` object is modified
     in-place.
 
-    :arg cols: Sequence of :class:`.Column` objects.
+    :arg cols:   Sequence of :class:`.Column` objects.
+    :arg suffix: String to append to the name of all renamed columns.
+                 Defaults to an empty string.
     """
 
+    if suffix is None:
+        suffix = ''
+
     counts = collections.defaultdict(list)
 
     for col in cols:
@@ -544,7 +560,7 @@ def renameDuplicateColumns(cols):
         counts[col.name].append(col)
         count = len(counts[col.name])
         if count > 1:
-            newname  = '{}.{}'.format(col.name, count - 1)
+            newname  = '{}.{}{}'.format(col.name, count - 1, suffix)
             col.name = newname
 
             log.warning('Duplicate column detected (%s: %s) - renamed to %s',
