@@ -23,11 +23,14 @@ UKBiobank showcase at https://biobank.ctsu.ox.ac.uk/crystal/schema.cgi.
 """
 
 
-import os.path   as op
-import functools as ft
+import os.path     as op
+import functools   as ft
+import dataclasses as dc
 
-import pandas    as pd
-import numpy     as np
+import pandas      as pd
+import numpy       as np
+
+from typing import Any
 
 
 HIERARCHY_DATA_NAMES = {
@@ -45,7 +48,38 @@ class CircularError(Exception):
     """Error raised by the :meth:`Hierarchy.parents` method in the event
     that a circular relationship is detected in a hierarchy.
     """
-    pass
+
+
+@ft.lru_cache()
+def loadEncodingTable():
+    """Loads the encoding.txt file built into funpack, which contains
+    information about all UK Biobank encodings.
+    """
+    datadir   = op.join(op.dirname(__file__), 'data')
+    encodings = op.join(datadir, 'encoding.txt')
+    encodings = pd.read_csv(encodings, sep='\t', index_col='encoding_id')
+    return encodings
+
+
+def dataCodingType(coding):
+    """Returns a data type suitable for representing values in the given
+    encoding.
+    """
+    encodings = loadEncodingTable()
+    typecode  = encodings.loc[coding, 'coded_as']
+
+    # the coded_as column conains a numeric ID
+    # for each encoding, denoting its type. The
+    # codes are the same as those used in
+    # field.txt (see loadtables.loadTableBases).
+    dtypes = {
+        11 : int,
+        21 : int,
+        31 : float,
+        41 : str,
+        51 : str
+    }
+    return dtypes.get(typecode, str)
 
 
 def getHierarchyFilePath(dtable=None,
@@ -102,32 +136,31 @@ def loadHierarchyFile(fname):
     ``fname`` is assumed to refer to a tab-separated file containing
     the following columns:
 
-      - ``coding``:    A variable value
+      - ``coding``:    A variable value (not necessarily unique)
       - ``meaning``:   Description
-      - ``node_id``:   Index in the hierarchy
-      - ``parent_id``: Index of this node's parent
+      - ``node_id``:   Unique numeric identifier for each node
+      - ``parent_id``: Identifier of each node's parent
 
-    It is assumed that all codings have a sequential ``node_id`` starting from
-    1, and that the parent node(s) have a ``parent_id`` of 0.
-    The codes are not assumed to be ordered by their ID.
+    It is assumed that all codings have a unique ``node_id``. Top-level parent
+    nodes (nodes with no parent of their own) often have an ID of 0, although
+    this is not assumed.
 
     :arg fname: File containing hierarchy information for one variable
     :returns:   A :class:`Hierarchy` object.
     """
 
-    data      = pd.read_csv(fname, delimiter='\t', index_col=False)
+    # get the encoding ID fromn "coding<id>.tsv",
+    # and look up a suitable data type
+    encoding  = int(op.basename(fname)[6:-4])
+    dtype     = {'coding' : dataCodingType(encoding)}
+
+    # load the hierarchy encoding file, ensuring
+    # that the coding values are converted
+    data      = pd.read_csv(fname, sep='\t', index_col=False, dtype=dtype)
     codings   = data['coding']   .values
     meanings  = data['meaning']  .values
     nodeIds   = data['node_id']  .values
     parentIds = data['parent_id'].values
-
-    order = np.argsort(nodeIds)
-
-    codings   = codings[  order]
-    meanings  = meanings[ order]
-    nodeIds   = nodeIds[  order] - 1
-    parentIds = parentIds[order] - 1
-
     return Hierarchy(nodeIds, parentIds, codings, meanings)
 
 
@@ -140,6 +173,10 @@ def codeToNumeric(code,
     """Converts a hierarchical code into a numeric version. See the
     :func:`getHierarchyFilePath` for information on the arguments.
 
+    Some hierarchical codings in the UKB do not have unique coding values for
+    parent/non-leaf nodes. If this function is passed such a value, ``np.nan``
+    is returned.
+
     :arg code:   Code to convert
     :arg name:   Data coding name
     :arg dtable: The :class:`.DataTable`
@@ -157,10 +194,7 @@ def codeToNumeric(code,
         hier = loadHierarchyFile(hier)
 
     try:
-        # hierarchy uses 0-indexing,
-        # whereas the hierarchy files
-        # use 1-indexing
-        return int(hier.index(code) + 1)
+        return int(hier.index(code))
     except KeyError:
         return np.nan
 
@@ -178,14 +212,25 @@ def numericToCode(code, name=None, dtable=None, vid=None, coding=None):
     hier = loadHierarchyFile(hier)
 
     try:
-        return hier.coding(int(code - 1))
+        return hier.coding(int(code))
     except KeyError:
         return 'NaN'
 
 
-class Hierarchy(object):
-    """The ``Hierarchy`` class allows information in a hierarchical variable to be
-    queried. The :meth:`parents` method will return all parents in the
+@dc.dataclass
+class Node:
+    """Represnts a node in a hierarchical encoding. Used by the
+    :class:`Hierarchy` class.
+    """
+    node_id:   int
+    parent_id: int
+    coding:    Any
+    attrs:     dict
+
+
+class Hierarchy:
+    """The ``Hierarchy`` class allows information in a hierarchical variable to
+    be queried. The :meth:`parents` method will return all parents in the
     hierarchy for a given value (a.k.a. *coding*), and the :meth:`description`
     method will return the description for a value.
 
@@ -193,60 +238,92 @@ class Hierarchy(object):
     :meth:`set` and :meth:`get` methods.
     """
 
+
     def __init__(self, nodes, parents, codings, descs):
         """Create a ``Hierarchy`` object.
 
-        :arg nodes:   Node IDs. Currently assumed to be equivalent to
-                      ``np.arange(len(nodes))``
+        :arg nodes:   Node IDs. Assumed to be unique.
         :arg parents: Parent IDs for each node.
         :arg codings: Value/coding for each node.
-        :arg descs:   Description for each node
+        :arg descs:   Description for each node.
         """
 
-        self.__nodes    = nodes
-        self.__parents  = parents
-        self.__codings  = codings
-        self.__attrs    = {}
-        self.__codeidxs = {}
+        # Mappings from node ID and from
+        # coding value to Node objects.
+        # Mappings for unique codings
+        # only are retained - non-unique
+        # mappings are culled afterwards.
+        self.__by_id     = {}
+        self.__by_coding = {}
+        counts           = {}
 
-        self.__attrs['description'] = descs
+        for i, node_id in enumerate(nodes):
+            coding    = codings[i]
+            parent_id = parents[i]
+            attrs     = {'description' : descs[i]}
+            node      = Node(node_id, parent_id, coding, attrs)
 
-        for i, c in enumerate(self.__codings):
-            self.__codeidxs[c] = i
+            self.__by_id[    node_id] = node
+            self.__by_coding[coding]  = node
+            counts[          coding]  = counts.get(coding, 0) + 1
+
+        for coding, count in counts.items():
+            if count > 1:
+                self.__by_coding.pop(coding)
 
 
     @property
     def codings(self):
-        """Return a list of all codings in the hierarchy. """
-        return list(self.__codings)
+        """Return a list of all unique codings in the hierarchy. """
+        return list(self.__by_coding.keys())
 
 
     def index(self, coding):
-        """Return the node ID for the given ``coding``. """
-        return self.__codeidxs[coding]
+        """Return the node ID for the given ``coding``. A ``KeyError``
+        is raised for non-uniqwue or invalid codings.
+        """
+        return self.__by_coding[coding].node_id
 
 
     def coding(self, nodeID):
         """Return the coding for the given ``nodeID``. """
-        return self.__codings[nodeID]
+        return self.__by_id[nodeID].coding
 
 
     def parents(self, coding):
-        """Return all parents of the given coding. """
+        """Return codings for all parents of the given coding.  A ``KeyError``
+        is raised for non-uniqwue or invalid codings.
+        """
+        pids    = self.parentIDs(self.__by_coding[coding].node_id)
+        codings = [self.__by_id[p].coding for p in pids]
+        return codings
 
-        parents = []
-        cidx    = self.index(coding)
-        pidx    = self.__parents[cidx]
-        seen    = set()
 
-        while pidx >= 0:
-            if pidx in seen:
-                raise CircularError(pidx + 1, str(self.coding(pidx)))
-            seen.add(pidx)
-            parents.append(self.coding(pidx))
-            pidx = self.__parents[pidx]
+    def parentIDs(self, nodeID):
+        """Return IDs of  all parents of the given node. """
 
-        return parents
+        node       = self.__by_id[nodeID]
+        parent     = node
+        parent_ids = []
+        seen       = set()
+
+        while True:
+
+            parent = self.__by_id.get(parent.parent_id, None)
+
+            # we're at the top of the tree
+            if parent is None:
+                break
+
+            # circular relationship in tree - should never
+            # happen, means the coding file is corrupt
+            if parent.node_id in seen:
+                raise CircularError(f'{parent.node_id} / {parent.coding}')
+
+            seen      .add(   parent.node_id)
+            parent_ids.append(parent.node_id)
+
+        return parent_ids
 
 
     def description(self, coding):
@@ -255,16 +332,14 @@ class Hierarchy(object):
 
 
     def get(self, coding, attr):
-        """Get the given attribue for the given coding. """
-        idx = self.index(coding)
-        return self.__attrs[attr][idx]
+        """Get the given attribute for the given coding. Returns a ``KeyError``
+        for non-unique / invalid codings, or non-existent attributes.
+        """
+        return self.__by_coding[coding].attrs[attr]
 
 
     def set(self, coding, attr, value):
-        """Set an attribute for the given coding. """
-
-        if attr not in self.__attrs:
-            self.__attrs[attr] = [None] * len(self.__nodes)
-
-        idx = self.index(coding)
-        self.__attrs[attr][idx] = value
+        """Set an attribute for the given coding. Returns a ``KeyError``
+        for non-unique / invalid codings.
+        """
+        self.__by_coding[coding].attrs[attr] = value
