@@ -17,5 +17,6 @@ from .core import   (importData,
                      MERGE_AXIS,
                      MERGE_STRATEGY,
                      MERGE_AXIS_OPTIONS,
-                     MERGE_STRATEGY_OPTIONS)  # noqa
-from .filter import (restrictVariables,)      # noqa
+                     MERGE_STRATEGY_OPTIONS)              # noqa
+from .filter import (restrictVariables,
+                     REMOVE_DUPLICATE_COLUMN_IDENTIFIER)  # noqa
