@@ -66,10 +66,13 @@ def importData(fileinfo,
                cattable,
                variables=None,
                colnames=None,
+               excludeColnames=None,
                categories=None,
                subjects=None,
                subjectExprs=None,
                exclude=None,
+               excludeVariables=None,
+               excludeCategories=None,
                trustTypes=False,
                mergeAxis=None,
                mergeStrategy=None,
@@ -89,70 +92,81 @@ def importData(fileinfo,
 
       3. Creates and returns a :class:`DataTable`.
 
-    :arg fileinfo:      :class:`.FileInfo` object describing the input file(s).
+    :arg fileinfo:          :class:`.FileInfo` object describing the input
+                            file(s).
 
-    :arg vartable:      The data coding table
+    :arg vartable:          The data coding table
 
-    :arg proctable:     The processing table
+    :arg proctable:         The processing table
 
-    :arg cattable:      The category table
+    :arg cattable:          The category table
 
-    :arg variables:     List of variable IDs to import
+    :arg variables:         List of variable IDs to import
 
-    :arg colnames:      List of names/glob-style wildcard patterns
-                        specifying columns to import.
+    :arg colnames:          List of names/glob-style wildcard patterns
+                            specifying columns to import.
 
-    :arg categories:    List of category names to import
+    :arg excludeColnames:   List of column name suffixes specifying columns
+                            to exclude.
 
-    :arg subjects:      List of subjects to include
+    :arg categories:        List of category names/IDs to import
 
-    :arg subjectExprs:  List of subject inclusion expressions
+    :arg subjects:          List of subjects to include
 
-    :arg exclude:       List of subjects to exclude
+    :arg subjectExprs:      List of subject inclusion expressions
 
-    :arg trustTypes:    If ``True``, it is assumed that columns with a
-                        known data type do not contain any bad/unparseable
-                        values. This improves performance, but will cause
-                        an error if the assumption does not hold.
+    :arg exclude:           List of subjects to exclude
 
-    :arg mergeAxis:     Merging axis to use when loading multiple data
-                        files - see the :func:`mergeData` function.
+    :arg excludeVariables:  List of variables to exclude
 
-    :arg mergeStrategy: Merging strategy to use when loading multiple
-                        data files - see the :func:`mergeData` function.
+    :arg excludeCategories: List of category names/IDs to exclude
 
-    :arg indexVisits:   Re-arrange the data so that rows are indexed by
-                        subject ID and visit, rather than visits being
-                        split into separate columns. Only applied to
-                        variables which are labelled with Instancing 2.
+    :arg trustTypes:        If ``True``, it is assumed that columns with a
+                            known data type do not contain any bad/unparseable
+                            values. This improves performance, but will cause
+                            an error if the assumption does not hold.
 
-    :arg dropNaRows:    If ``True``, rows which do not contain data for any
-                        columns are not loaded.
+    :arg mergeAxis:         Merging axis to use when loading multiple data
+                            files - see the :func:`mergeData` function.
 
-    :arg njobs:         Number of processes to use for parallelising tasks.
+    :arg mergeStrategy:     Merging strategy to use when loading multiple
+                            data files - see the :func:`mergeData` function.
 
-    :arg mgr:           :class:`multiprocessing.Manager` object for
-                        parallelisation
+    :arg indexVisits:       Re-arrange the data so that rows are indexed by
+                            subject ID and visit, rather than visits being
+                            split into separate columns. Only applied to
+                            variables which are labelled with Instancing 2.
 
-    :arg dryrun:        If ``True`` the data is not loaded.
+    :arg dropNaRows:        If ``True``, rows which do not contain data for any
+                            columns are not loaded.
 
-    :returns:           A tuple containing:
+    :arg njobs:             Number of processes to use for parallelising tasks.
 
-                         - A :class:`DataTable`, which contains references
-                           to the data, and the variable and procesing
-                           tables.
+    :arg mgr:               :class:`multiprocessing.Manager` object for
+                            parallelisation
 
-                         - A list of :class:`.Column` objects that were not
-                           loaded from each input file.
+    :arg dryrun:            If ``True`` the data is not loaded.
+
+    :returns:               A tuple containing:
+
+                             - A :class:`DataTable`, which contains references
+                               to the data, and the variable and procesing
+                               tables.
+
+                             - A list of :class:`.Column` objects that were not
+                               loaded from each input file.
     """
 
-    variables = filter.restrictVariables(cattable, variables, categories)
+    variables, excludevars = filter.restrictVariables(
+        cattable, variables, categories, excludeVariables, excludeCategories)
 
     # Figure out which columns to load
     cols, drop = filter.columnsToLoad(fileinfo,
                                       vartable,
                                       variables,
-                                      colnames)
+                                      excludevars,
+                                      colnames,
+                                      excludeColnames)
 
     # Load those columns, merging
     # multiple input files.
