@@ -43,10 +43,15 @@ import              logging
 import              warnings
 import              collections
 
+from typing import Tuple, Sequence, Union, Dict, List, Type, Any, Callable
+from typing_extensions import Literal
+
 import numpy  as np
 import pandas as pd
 
 import funpack.util       as util
+import funpack.fileinfo   as finfo
+import funpack.datatable  as datatable
 import funpack.processing as processing
 import funpack.expression as expression
 
@@ -54,7 +59,7 @@ import funpack.expression as expression
 log = logging.getLogger(__name__)
 
 
-def convert_type(val):
+def convert_type(val : str) -> util.CTYPES:
     """Convert a string containing a UK BioBank type into a numerical
     identifier for that type - see :attr:`funpack.util.CTYPES`.
     """
@@ -86,7 +91,7 @@ def convert_type(val):
     return valmap.get(val.lower(), util.CTYPES.unknown)
 
 
-def convert_dtype(val):
+def convert_dtype(val : str) -> Union[np.dtype, Literal[np.nan]]:
     """Convert a string containing a ``numpy.dtype`` (e.g. ``'float32'``)
     into a ``dtype`` object.
     """
@@ -102,7 +107,7 @@ def convert_dtype(val):
     return dtype
 
 
-def convert_comma_sep_text(val):
+def convert_comma_sep_text(val : str) -> Union[List[str], Literal[np.nan]]:
     """Convert a string containing comma-separated text into a list. """
     if val.strip() == '':
         return np.nan
@@ -110,7 +115,7 @@ def convert_comma_sep_text(val):
     return [w.strip() for w in words]
 
 
-def convert_comma_sep_numbers(val):
+def convert_comma_sep_numbers(val : str) -> Union[np.ndarray, Literal[np.nan]]:
     """Convert a string containing comma-separated numbers into a ``numpy``
     array.
     """
@@ -119,7 +124,8 @@ def convert_comma_sep_numbers(val):
     return np.fromstring(val, sep=',', dtype=np.float)
 
 
-def convert_ParentValues(val):
+def convert_ParentValues(
+        val : str) -> Union[List[expression.Expression], Literal[np.nan]]:
     """Convert a string containing a sequence of comma-separated
     ``ParentValue`` expressions into a sequence of :class:`.Expression`
     objects.
@@ -129,7 +135,7 @@ def convert_ParentValues(val):
     return [expression.Expression(e) for e in val.split(',')]
 
 
-def convert_Process_Variable(val):
+def convert_Process_Variable(val : str) -> Tuple[str, List[int]]:
     """Convert a string containing a process variable specification - one of:
 
       - One or more comma-separated MATLAB-style ``start:stop:step`` ranges,
@@ -168,7 +174,7 @@ def convert_Process_Variable(val):
     if len(tokens) == 1 and \
        tokens[0] in ('all', 'all_independent',
                      'all_except', 'all_independent_except'):
-        return tokens[0], []
+        return np.array((tokens[0], []), dtype=object)
 
     if tokens[0] in ('independent', 'all_except', 'all_independent_except'):
         ptype  = tokens[0]
@@ -176,10 +182,15 @@ def convert_Process_Variable(val):
     else:
         ptype = 'vids'
 
-    return ptype, list(it.chain(*[util.parseMatlabRange(t) for t in tokens]))
+    vids = list(it.chain(*[util.parseMatlabRange(t) for t in tokens]))
+
+    return np.array((ptype, vids), dtype=object)
 
 
-def convert_Process(ptype, val):
+def convert_Process(
+        ptype : str,
+        val   : str
+) -> Dict[str, processing.Process]:
     """Convert a string containing a sequence of comma-separated ``Process`` or
     ``Clean`` expressions into an ``OrderedDict`` of :class:`.Process`
     objects (with the process names used as dictionary keys).
@@ -192,7 +203,7 @@ def convert_Process(ptype, val):
     return collections.OrderedDict([(p.name, p)  for p in procs])
 
 
-def convert_category_variables(val):
+def convert_category_variables(val : str) -> List[int]:
     """Convert a string containing a sequence of comma-separated variable IDs
     or ranges into a list of variable IDs. Variables may be specified as
     integer IDs, or via a MATLAB-style ``start:step:stop`` range. See
@@ -272,11 +283,7 @@ VARTABLE_DTYPES = {
 VARTABLE_CONVERTERS = {
     'Type'         : convert_type,
     'InternalType' : convert_dtype,
-    'NAValues'     : convert_comma_sep_numbers,
-    'RawLevels'    : convert_comma_sep_numbers,
-    'NewLevels'    : convert_comma_sep_numbers,
     'ParentValues' : convert_ParentValues,
-    'ChildValues'  : convert_comma_sep_numbers,
     'Clean'        : ft.partial(convert_Process, 'cleaner'),
 }
 """Custom converter functinos to use for some columns in the variable
@@ -293,15 +300,6 @@ DCTABLE_DTYPES = {
 """Types to use for some columns in the data coding table. """
 
 
-DCTABLE_CONVERTERS = {
-    'NAValues'  : convert_comma_sep_numbers,
-    'RawLevels' : convert_comma_sep_numbers,
-    'NewLevels' : convert_comma_sep_numbers,
-}
-"""Custom converter functinos to use for some columns in the data coding
-table.
-"""
-
 
 TYPETABLE_DTYPES = {
     'Type'  : object,
@@ -351,13 +349,19 @@ call to :func:`addImplicitCategories`).
 """
 
 
-def loadTables(fileinfo,
-               varfiles=None,
-               dcfiles=None,
-               typefile=None,
-               procfile=None,
-               catfile=None,
-               **kw):
+def loadTables(
+        fileinfo : finfo.FileInfo,
+        varfiles : Sequence[str] = None,
+        dcfiles  : Sequence[str] = None,
+        typefile : str           = None,
+        procfile : str           = None,
+        catfile  : str           = None,
+        **kw
+) -> Tuple[pd.DataFrame,
+           pd.DataFrame,
+           pd.DataFrame,
+           List[datatable.Column],
+           List[datatable.Column]]:
     """Loads the data tables used to run ``funpack``.
 
     :arg fileinfo:  :class:`.FileInfo` object describing the input data files.
@@ -401,19 +405,23 @@ def loadTables(fileinfo,
     return vartable, proctable, cattable, unk, unc
 
 
-def loadVariableTable(fileinfo,
-                      varfiles=None,
-                      dcfiles=None,
-                      typefile=None,
-                      noBuiltins=False,
-                      naValues=None,
-                      childValues=None,
-                      recoding=None,
-                      clean=None,
-                      typeClean=None,
-                      globalClean=None,
-                      dropAbsent=True,
-                      **kwargs):
+def loadVariableTable(
+        fileinfo    : finfo.FileInfo,
+        varfiles    : Sequence[str]              = None,
+        dcfiles     : Sequence[str]              = None,
+        typefile    : str                        = None,
+        noBuiltins  : bool                       = False,
+        naValues    : Dict[int, str]             = None,
+        childValues : Dict[int, Tuple[str, str]] = None,
+        recoding    : Dict[int, Tuple[str, str]] = None,
+        clean       : Dict[int, str]             = None,
+        typeClean   : Dict[util.CTYPES, str]     = None,
+        globalClean : str                        = None,
+        dropAbsent  : bool                       = True,
+        **kwargs  # pylint: disable=unused-argument
+) -> Tuple[pd.DataFrame,
+           Sequence[datatable.Column],
+           Sequence[datatable.Column]]:
     """Given variable table and datacoding table file names, builds and returns
     the variable table.
 
@@ -429,30 +437,37 @@ def loadVariableTable(fileinfo,
     :arg noBuiltins:  If provided, the built-in variable and datacoding base
                       tables are not loaded.
 
-    :arg naValues:    Dictionary of ``{vid : [values]}`` mappings, specifying
-                      values which should be replaced with NA.
+    :arg naValues:    Dictionary of ``{vid : values}`` mappings, specifying
+                      values which should be replaced with NA. The values
+                      are expected to be strings of comma-separated values.
 
-    :arg childValues: Dictionary of ``{vid : [exprs], [values]}`` mappings,
+    :arg childValues: Dictionary of ``{vid : (exprs, values)}`` mappings,
                       specifying parent value expressions, and corresponding
-                      child values.
+                      child values. The expressions and values
+                      are expected to be strings of comma-separated values
+                      of the same length.
 
-    :arg recoding:    Dictionary of ``{vid : [rawlevel], [newlevel]}``
-                      mappings
+    :arg recoding:    Dictionary of ``{vid : (rawlevel, newlevel)}``
+                      mappings. The raw and enw levels are expected to be
+                      strings of comma-separated values of the same length.
 
     :arg clean:       Dictionary of ``{vid : expr}`` mappings containing
                       cleaning functions to apply - this will override
                       any cleaning specified in the variable file, and
-                      any cleaning specified in ``typeClean``.
+                      any cleaning specified in ``typeClean``. The expressions
+                      are expected to be strings.
 
     :arg typeClean:   Dictionary of ``{type : expr}`` mappings containing
                       cleaning functions to apply to all variables of a
                       specific type - this will override any cleaning
-                      specified in the type file.
+                      specified in the type file. The expressions
+                      are expected to be strings.
 
     :arg globalClean: Expression containing cleaning functions to
                       apply to every variable - this will be performed after
                       variable-specific cleaning in the variable table,
-                      or specified via ``clean`` or ``typeClean``.
+                      or specified via ``clean`` or ``typeClean``. The
+                      expressions are expected to be strings.
 
     :arg dropAbsent:  If ``True`` (the default), remove all variables from the
                       variable table which are not present in the data
@@ -486,7 +501,7 @@ def loadVariableTable(fileinfo,
                                dcfiles,
                                'data coding',
                                DCTABLE_DTYPES,
-                               DCTABLE_CONVERTERS,
+                               {},
                                DCTABLE_COLUMNS)
     tytable  = mergeTableFiles(None,
                                [typefile],
@@ -533,29 +548,56 @@ def loadVariableTable(fileinfo,
     # table (overriding whatever was specified
     # in the datacoding/variable tables)
     if naValues is not None:
-        naValues = {vid : np.array(vals) for vid, vals in naValues.items()}
         mergeIntoVariableTable(
             vartable,
             'NAValues',
             naValues)
 
     if recoding is not None:
-        recoding = {vid : (np.array(raw), np.array(new))
-                    for vid, (raw, new) in recoding.items()}
         mergeIntoVariableTable(
             vartable,
             ['RawLevels', 'NewLevels'],
             recoding)
 
     if childValues is not None:
-        childValues = {vid : (convert_ParentValues(expr),
-                              np.array(values))
+        childValues = {vid : (convert_ParentValues(expr), values)
                        for vid, (expr, values) in childValues.items()}
         mergeIntoVariableTable(
             vartable,
             ['ParentValues', 'ChildValues'],
             childValues)
 
+    # navalues, raw/new levels and child values
+    # are all still comma-separated strings -
+    # convert them to types appropriate to the
+    # datafield/variable
+    def convert(rowvalues, column):
+        val = rowvalues[column]
+
+        if pd.isna(val):
+            return np.nan
+
+        val   = convert_comma_sep_text(val)
+        ctype = rowvalues['Type']
+        dtype = util.DATA_TYPES.get(ctype, None)
+
+        if ctype in (util.CTYPES.date, util.CTYPES.time):
+            return pd.to_datetime(val).values
+        else:
+            return np.array(val, dtype=dtype)
+
+    # Make sure the series type stays as "object" - if
+    # no rules are specified on any vid, the convert
+    # function will return all nans, and pandas will
+    # coerce the series type to float64, which may
+    # interfere with subsequent rule assignments (as
+    # these series may contain numpy arrays or
+    # list-likes)
+    if len(vartable) > 0:
+        for col in ('NAValues', 'RawLevels', 'NewLevels', 'ChildValues'):
+            newseries     = vartable.apply(convert, axis=1, args=(col,))
+            vartable[col] = newseries.astype(object)
+
     # Before merging the cleaning functions
     # in, we generate a list of variables
     # which are "uncleaned", i.e. have not
@@ -611,7 +653,7 @@ def loadVariableTable(fileinfo,
     return vartable, unknownVars, uncleanVars
 
 
-def loadTableBases():
+def loadTableBases() -> Tuple[pd.DataFrame, pd.DataFrame]:
     """Loads the UK Biobank variable and data coding schema files.
 
     This function is called by :func:`loadVariableTable`. It loads the UK
@@ -677,7 +719,7 @@ def loadTableBases():
 
     dcbase  = pd.DataFrame({'ID' : encodings['encoding_id']}).set_index('ID')
     varbase = pd.DataFrame({
-        'ID'           : fields['field_id'],
+        'ID'           : fields['field_id'].astype(np.uint64),
         'Type'         : fields['value_type'].combine(fields['base_type'],
                                                       settype),
         'Description'  : fields['title'],
@@ -691,7 +733,14 @@ def loadTableBases():
     return varbase, dcbase
 
 
-def mergeTableFiles(base, fnames, what, dtypes, converters, columns):
+def mergeTableFiles(
+        base       : pd.DataFrame,
+        fnames     : List[str],
+        what       : str,
+        dtypes     : Dict[str, Type],
+        converters : Dict[str, Callable],
+        columns    : List[str]
+) -> pd.DataFrame:
     """Load and merge one or more table files.
 
     This function is called by :func:`loadVariableTable` to load the variable,
@@ -736,7 +785,8 @@ def mergeTableFiles(base, fnames, what, dtypes, converters, columns):
 
             log.debug('Loading %s table from %s', what, f)
 
-            table = pd.read_csv(f, '\t',
+            table = pd.read_csv(f,
+                                sep='\t',
                                 index_col=0,
                                 dtype=dtypes,
                                 converters=converters)
@@ -759,14 +809,12 @@ def mergeTableFiles(base, fnames, what, dtypes, converters, columns):
             # non-na values in later files take
             # precedence.
             for c in [c[:-2] for c in merged.columns if c.endswith('_x')]:
-                bcolname        = c + '_x'
-                tcolname        = c + '_y'
-                bcol            = merged[bcolname]
-                tcol            = merged[tcolname]
-                notna           = tcol.notna()
-                bcol.loc[notna] = tcol[notna]
-                merged[c]       = bcol
-                merged          = merged.drop(columns=[bcolname, tcolname])
+                bname                = c + '_x'
+                tname                = c + '_y'
+                merged[c]            = merged[bname]
+                notna                = merged[tname].notna()
+                merged.loc[notna, c] = merged.loc[notna, tname]
+                merged               = merged.drop(columns=[bname, tname])
             base = merged
 
     # no base, and no files
@@ -790,7 +838,11 @@ def mergeTableFiles(base, fnames, what, dtypes, converters, columns):
     return base
 
 
-def sanitiseVariableTable(vartable, cols, dropAbsent):
+def sanitiseVariableTable(
+        vartable   : pd.DataFrame,
+        cols       : Sequence[datatable.Column],
+        dropAbsent : bool
+) -> List[datatable.Column]:
     """Ensures that the variable table contains an entry for every
     variable in the input data.
 
@@ -836,7 +888,10 @@ def sanitiseVariableTable(vartable, cols, dropAbsent):
     return unknownVars
 
 
-def mergeIntoVariableTable(vartable, cols, mapping):
+def mergeIntoVariableTable(
+        vartable : pd.DataFrame,
+        cols     : Sequence[str],
+        mapping  : Union[str, Dict[int, Any]]):
     """Merge data from ``mapping`` into the variable table.
 
     Called by :func:`loadVariableTable`.
@@ -869,7 +924,9 @@ def mergeIntoVariableTable(vartable, cols, mapping):
             vartable.at[vid, col] = val
 
 
-def mergeDataCodingTable(vartable, dctable):
+def mergeDataCodingTable(
+        vartable : pd.DataFrame,
+        dctable  : pd.DataFrame):
     """Merges information from the data coding table into the variable
     table.
 
@@ -891,7 +948,12 @@ def mergeDataCodingTable(vartable, dctable):
         vartable.loc[mask, field] = newvals
 
 
-def mergeCleanFunctions(vartable, tytable, clean, typeClean, globalClean):
+def mergeCleanFunctions(
+        vartable    : pd.DataFrame,
+        tytable     : pd.DataFrame,
+        clean       : Dict[int, str],
+        typeClean   : Dict[str, str],
+        globalClean : str):
     """Merges custom clean functions into the variable table.
 
     Called by :func:`loadVariableTable`.
@@ -964,7 +1026,12 @@ def mergeCleanFunctions(vartable, tytable, clean, typeClean, globalClean):
             else:             vartable.loc[ vid,  'Clean'].update(gpp)
 
 
-def addNewVariable(vartable, vid, name, dtype=None, instancing=None):
+def addNewVariable(
+        vartable   : pd.DataFrame,
+        vid        : int,
+        name       : str,
+        dtype      : np.dtype = None,
+        instancing : int      = None):
     """Add a new row to the variable table.
 
     The ``instancing`` argument defines the meaning of the
@@ -1021,11 +1088,13 @@ def addNewVariable(vartable, vid, name, dtype=None, instancing=None):
     vartable.loc[vid, 'Instancing']  = instancing
 
 
-def loadProcessingTable(procfile=None,
-                        skipProcessing=False,
-                        prependProcess=None,
-                        appendProcess=None,
-                        **kwargs):
+def loadProcessingTable(
+        procfile       : str                             = None,
+        skipProcessing : bool                            = False,
+        prependProcess : Sequence[Tuple[List[int], str]] = None,
+        appendProcess  : Sequence[Tuple[List[int], str]] = None,
+        **kwargs  # pylint: disable=unused-argument
+) -> pd.DataFrame:
     """Loads the processing table from the given file.
 
     :arg procfile:       Path to the processing table file.
@@ -1050,7 +1119,8 @@ def loadProcessingTable(procfile=None,
 
     if (procfile is not None) and (not skipProcessing):
         log.debug('Loading processing table from %s', procfile)
-        proctable = pd.read_csv(procfile, '\t',
+        proctable = pd.read_csv(procfile,
+                                sep='\t',
                                 index_col=False,
                                 skip_blank_lines=True,
                                 comment='#',
@@ -1065,19 +1135,21 @@ def loadProcessingTable(procfile=None,
     for i, (vids, procs) in enumerate(prependProcess):
         vids  = convert_Process_Variable(vids)
         procs = convert_Process('processor', procs)
-        proctable.loc[i, ['Variable', 'Process']] = [vids, procs]
+        proctable.at[i, 'Variable'] = np.array(vids, dtype=object)
+        proctable.at[i, 'Process']  = procs
 
     for i, (vids, procs) in enumerate(appendProcess, len(proctable.index)):
         vids  = convert_Process_Variable(vids)
         procs = convert_Process('processor', procs)
-        proctable.loc[i, ['Variable', 'Process']] = [vids, procs]
+        proctable.at[i, 'Variable'] = np.array(vids, dtype=object)
+        proctable.at[i, 'Process']  = procs
 
     proctable.sort_index(inplace=True)
 
     return proctable
 
 
-def loadCategoryTable(catfile=None):
+def loadCategoryTable(catfile : str = None) -> pd.DataFrame:
     """Loads the category table from the given file.
 
     :arg catfile: Path to the category file.
@@ -1085,7 +1157,7 @@ def loadCategoryTable(catfile=None):
     if catfile is not None:
         log.debug('Loading category table from %s', catfile)
         cattable = pd.read_csv(catfile,
-                               '\t',
+                               sep='\t',
                                index_col=0,
                                dtype=CATTABLE_DTYPES,
                                converters=CATTABLE_CONVERTERS)
@@ -1097,14 +1169,17 @@ def loadCategoryTable(catfile=None):
     return cattable
 
 
-def categoryVariables(cattable, categories):
+def categoryVariables(
+        cattable   : pd.DataFrame,
+        categories : Sequence[Union[int, str]]
+) -> List[int]:
     """Returns a list of variable IDs from ``cattable`` which correspond to
     the strings in ``categories``.
 
     :arg cattable:   The category table.
     :arg categories: Sequence of integer category IDs or label sub-strings
                      specifying the categories to return.
-    :returns:        A list of variable IDs as strings.
+    :returns:        A list of variable IDs.
     """
 
     allvars = []
@@ -1128,7 +1203,31 @@ def categoryVariables(cattable, categories):
     return allvars
 
 
-def addImplicitCategories(cattable, unknown, uncat):
+def variableCategories(
+        cattable : pd.DataFrame,
+        vids     : Sequence[int]
+) -> Dict[int, List[str]]:
+    """Identify the categories for each variable in vids.
+
+    :arg cattable: The category table
+    :arg vids:     Sequence of variable/datafield IDs
+    :returns:      A dict of ``{vid : [category]}`` mappings
+    """
+
+    categories = collections.defaultdict(list)
+
+    for v in vids:
+        for i in cattable.index:
+            if v in cattable.loc[i, 'Variables']:
+                categories[v].append(cattable.loc[i, 'Category'])
+
+    return categories
+
+
+def addImplicitCategories(
+        cattable : pd.DataFrame,
+        unknown  : Sequence[datatable.Column],
+        uncat    : Sequence[datatable.Column]):
     """Adds some implicit/automatic categories to the category table.
 
     The following implicit categories are added:
@@ -1169,7 +1268,10 @@ def addImplicitCategories(cattable, unknown, uncat):
         cattable.loc[idx, 'Variables'] = list(vids)
 
 
-def columnTypes(vartable, columns):
+def columnTypes(
+        vartable : pd.DataFrame,
+        columns  : Sequence[datatable.Column]
+) -> Tuple[List[util.CTYPES], Dict[str, np.dtype]]:
     """Retrieves the type of each column in ``cols`` as listed in ``vartable``.
     Also identifies a suitable internal data type to use for each column where
     possible.
@@ -1214,7 +1316,10 @@ def columnTypes(vartable, columns):
     return vttypes, dtypes
 
 
-def identifyUncategorisedVariables(fileinfo, cattable):
+def identifyUncategorisedVariables(
+        fileinfo : finfo.FileInfo,
+        cattable : pd.DataFrame
+) -> List[datatable.Column]:
     """Called by :func:`loadTables`. Identifies all variables which are in the
     data file(s), but which are uncategorised (not present in any categories
     in the category table).
