@@ -110,12 +110,23 @@ def main(argv=None):
 
     if args.num_jobs > 1:
         log.debug('Running up to %i jobs in parallel', args.num_jobs)
-        mgr = mp.Manager()
 
-        # We need to initialise icd10
-        # before the worker processes
-        # are created, so its state is
-        # shared by all processes.
+        # Funpack relies on fork() to share
+        # the dataset to child processes
+        # when parallelising tasks. This
+        # is potentially dangerous on macOS
+        # - see the "macOS issues" note in
+        # README, and the python bug report:
+        # https://bugs.python.org/issue33725
+        if mp.get_start_method(True) is None:
+            mp.set_start_method('fork')
+
+        # The icd10 module maintains information
+        # which is potentially shared (read from
+        # and written to) by multiple processes,
+        # so we use a mp.Manager to handle
+        # shared state.
+        mgr = mp.Manager()
         icd10.initialise(mgr)
     else:
         mgr = None
@@ -157,11 +168,25 @@ def doImport(args, mgr):
                   each input file.
     """
 
+    # if --remove_duplicates, we append
+    # an identifying suffix to the names
+    # of columns to be removed. This is
+    # then passed through as an exclusion
+    # pattern to the importData function
+    # via its excludeColnames option.
+    if args.remove_duplicates:
+        suffix           = importing.REMOVE_DUPLICATE_COLUMN_IDENTIFIER
+        renameDuplicates = True
+    else:
+        suffix           = None
+        renameDuplicates = args.rename_duplicates
+
     finfo = fileinfo.FileInfo(args.infile,
                               indexes=args.index,
                               loaders=args.loader,
                               encodings=args.encoding,
-                              renameDuplicates=args.rename_duplicates)
+                              renameDuplicates=renameDuplicates,
+                              renameSuffix=suffix)
 
     with util.timed('Table import', log):
         vartable, proctable, cattable, unknowns, uncategorised = \
@@ -183,10 +208,10 @@ def doImport(args, mgr):
                 prependProcess=args.prepend_process,
                 appendProcess=args.append_process)
 
+
     subjects, exprs = args.subject
-    variables       = args.variable
-    categories      = args.category
-    columns         = args.column
+    if suffix is None: excludeColnames = []
+    else:              excludeColnames = [suffix]
 
     # Import data
     with util.timed('Data import', log):
@@ -195,12 +220,15 @@ def doImport(args, mgr):
             vartable=vartable,
             proctable=proctable,
             cattable=cattable,
-            variables=variables,
-            colnames=columns,
-            categories=categories,
+            variables=args.variable,
+            colnames=args.column,
+            excludeColnames=excludeColnames,
+            categories=args.category,
             subjects=subjects,
             subjectExprs=exprs,
             exclude=args.exclude,
+            excludeVariables=args.exclude_variable,
+            excludeCategories=args.exclude_category,
             trustTypes=args.trust_types,
             mergeAxis=args.merge_axis,
             mergeStrategy=args.merge_strategy,
