@@ -117,14 +117,15 @@ multi-core environment.
 """
 
 
-import functools as ft
-import itertools as it
-import os.path   as op
-import              os
-import              glob
-import              logging
-import              tempfile
-import              collections
+import functools       as ft
+import itertools       as it
+import os.path         as op
+import                    os
+import                    glob
+import                    logging
+import                    tempfile
+import                    collections
+import collections.abc as abc
 
 import pyparsing as pp
 import pandas    as pd
@@ -605,8 +606,8 @@ class Process:
             else:
                 continue
 
-            if isinstance(val, collections.Sequence): auxvids.extend(val)
-            else:                                     auxvids.append(val)
+            if isinstance(val, abc.Sequence): auxvids.extend(val)
+            else:                             auxvids.append(val)
 
         return auxvids
 
