@@ -95,18 +95,48 @@ def removeIfSparse(
         naval      : Optional[Any]   = None,
         ignoreType : bool            = False
 ) -> List[datatable.Column]:
-    """removeIfSparse([minpres], [minstd], [mincat], [maxcat], [abspres], [abscat], [naval])
+    """removeIfSparse([minpres][, minstd][, mincat][, maxcat][, abspres][, abscat][, naval])
     Removes columns deemed to be sparse.
 
-    Removes columns for the variables in ``vids`` if they are sparse.
+    Removes columns for all specified variables if they fail a sparsity test.
+    The test is based on the following criteria:
 
-    :arg ignoreType: Defaults to ``False``. If ``True``, all specified tests are
-                     run regardless of the types of the ``vids``. Only used for
-                     testing.
+     - The number/proportion of non-NA values must be greater than
+       or equal to ``minpres``.
 
-    See the :func:`.isSparse` function for details on the other arguments.
+     - The standard deviation of the data must be greater than ``minstd``.
+
+     - For integer and categorical types, the number/proportion of the largest
+       category must be greater than ``mincat``.
+
+     - For integer and categorical types, the number/proportion of the largest
+       category must be less than ``maxcat``.
+
+    If **any** of these criteria are **not met**, the data is considered to be
+    sparse. Each criteria can be disabled by passing in ``None`` for the
+    relevant parameter.
+
+    The ``minstd`` test is only performed on numeric columns, and the
+    ``mincat``/``maxcat`` tests are only performed on integer/categorical
+    columns.
+
+    If ``abspres=True`` (the default), ``minpres`` is interpreted as an
+    absolute count. If ``abspress=False``, ``minpres`` is interpreted as a
+    proportion.
+
+    Similarly, If ``abscat=True`` (the default), ``mincat`` and ``maxcat`` are
+    interpreted as absolute counts. Otherwise ``mincat`` and ``maxcat`` are
+    interpreted as proportions
+
+    The ``naval`` argument can be used to customise the value to consider as
+    "missing" - it defaults to ``np.nan``.
     """  # noqa
 
+    # :arg ignoreType: Defaults to ``False``. If ``True``, all specified tests
+    #                  are run regardless of the types of the ``vids``. Only
+    #                  used for testing.
+
+
     cols   = []
     series = []
     vtypes = []
@@ -155,27 +185,41 @@ def removeIfSparse(
 def removeIfRedundant(dtable       : datatable.DataTable,
                       vids         : List[int],
                       corrthres    : float,
-                      nathres      : Optional[float] = None,
-                      pairwise     : Optional[bool]  = False,
-                      skipUnknowns : Optional[bool]  = False):
-    """removeIfRedundant(corrthres, [nathres], [pairwise])
+                      nathres      : float = None,
+                      skipUnknowns : bool  = False,
+                      precision    : str   = None,
+                      pairwise     : bool  = False):
+    """removeIfRedundant(corrthres, [nathres])
     Removes columns deemed to be redundant.
 
-    Removes columns from the variables in ``vids`` if they are redundant.
+    Removes columns from the specified group of variables if they are found to
+    be redundant with any other columns in the group.
 
-    Redundancy is determined by calculating the correlation between pairs
-    of columns using `corrthres` and `nathres` - see the :func:`.isRedundant`
-    function.
+    Redundancy is determined by calculating the correlation between all pairs
+    of columns - columns with an absolute correlation greater than
+    ``corrthres`` are identified as redundant.
 
-    If ``pairwise`` is ``True``, an alternative implementation is used which
-    may be faster on data sets with high missingness correlation.
+    The test can optionally take the patterns oof missing values into account
+    - if ``nathres`` is provided, the missingness correlation is also
+    calculated between all column pairs. Columns must have absolute
+    correlation greater than ``corrthres`` **and** absolute missingness
+    correlation greater than ``nathres`` to be identified as redundant.
 
-    :arg pairwise:     Use alternative pairwise implementation.
+    The ``skipUnknowns`` option defaults to ``False``.  If it is set to
+    ``True``, columns which are deemed to be redundant with respect to an
+    unknown or uncategorised column are **not** dropped.
 
-    :arg skipUnknowns: If ``True``, columns which are deemed to be redundant
-                       with respect to an unknown or uncategorised column
-                       are *not* dropped.
+    The ``precision`` option can be set to either ``'double'`` (the default)
+    or ``'single'`` - this controls whether 32 bit (single) or 64 bit (double)
+    precision floating point is used for the correlation calculation. Double
+    precision is recommended, as the correlation calculation algorithm can
+    be unstable for data with large values (>10e5).
     """
+    # :arg pairwise: Use alternative pairwise implementation. If ``pairwise``
+    #                is ``True``, an alternative implementation is used which
+    #                may be faster on data sets with high missingness
+    #                correlation.
+
 
     # Ignore non-numeric columns
     cols     = list(it.chain(*[dtable.columns(v) for v in vids]))
@@ -189,7 +233,7 @@ def removeIfRedundant(dtable       : datatable.DataTable,
                 dtable, data, corrthres, nathres)
         else:
             redundant = _removeIfRedundant(
-                dtable, data, corrthres, nathres)
+                dtable, data, corrthres, nathres, precision)
 
     redundant = util.dedup(sorted(redundant))
 
@@ -216,7 +260,7 @@ def removeIfRedundant(dtable       : datatable.DataTable,
     return [cols[r[0]] for r in redundant]
 
 
-def _removeIfRedundant(dtable, data, corrthres, nathres=None):
+def _removeIfRedundant(dtable, data, corrthres, nathres=None, precision=None):
     """Default fast implementation of redundancy check. Used when the
     ``pairwise`` option to :func:`removeIfRedundant` is ``False``.
 
@@ -225,12 +269,13 @@ def _removeIfRedundant(dtable, data, corrthres, nathres=None):
     :arg corrthres: Correlation threshold - see :func:`.redundantColumns`.
     :arg nathres:   Missingness correlation threshold - see
                     :func:`.redundantColumns`.
+    :arg precision: Floating point precision -``'single'`` or ``'double'``.
     :returns:       Sequence of tuples of column indices, where each tuple
                     ``(a, b)`` indicates that column ``a`` is redundant with
                     respect to column ``b``.
     """
 
-    return core.matrixRedundantColumns(data, corrthres, nathres)
+    return core.matrixRedundantColumns(data, corrthres, nathres, precision)
 
 
 def _pairwiseRemoveIfRedundant(dtable, data, corrthres, nathres=None):
@@ -349,52 +394,27 @@ def binariseCategorical(dtable,
                         take=None,
                         fillval=None,
                         replaceTake=True):
-    """binariseCategorical([acrossVisits], [acrossInstances], [minpres], [nameFormat], [replace])
+    """binariseCategorical([acrossVisits][, acrossInstances][, minpres][, nameFormat][, replace][, take][, fillval][, replaceTake])
     Replace a categorical column with one binary column per category.
 
-    Binarises categorical variables - replaces their columns with
-    one new column for each value, containing ``1`` for subjects
-    with that value, and ``0`` otherwise.
+    Binarises categorical variables - replaces their columns with one new
+    column for each unique value, containing ``1`` for subjects with that
+    value, and ``0`` otherwise. Thos procedure is applied independently to all
+    variables that are specified.
 
-    The :func:`.processing_functions_core.binariseCategorical` function is called
-    for each variable in ``vids`` - it internally parallelises generation of the
-    new columns using multiprocessing.
+    The ``acrossVisits`` option controls whether the binarisation is applied
+    across visits for each variable. It defaults to ``False``, meaning that
+    the binarisation is applied separately to the columns within each
+    visit. If set to ``True``, the binarisation will be applied to the columns
+    for all visits. Similarly, the ``acrossInstances`` option controls whether
+    the binarisation is applied across instances. This defaults to ``True``,
+    which is usually desirable - for example, data field `41202
+    <https://biobank.ctsu.ox.ac.uk/crystal/field.cgi?id=41202>`_ contains
+    multiple ICD10 diagnoses, separated across different instances.
 
-    :arg dtable:          The :class:`.DataTable`
-
-    :arg vids:            Sequence of variable IDs to (independently) apply the
-                          binarisation to.
-
-    :arg acrossVisits:    If ``True``, the binarisation is applied across
-                          visits for each variable.
-
-    :arg acrossInstances: If ``True``, the binarisation is applied across
-                          instances for each variable.
-
-    :arg minpres:         Optional threshold - categorical values with less
-                          than this many occurrences will not be added as
-                          columns.
-
-    :arg nameFormat:      Format string defining how the new columns should
-                          be named - see below.
-
-    :arg replace:         If ``True`` (the default), the original columns are
-                          returned for removal.
-
-    :arg take:            Optional variable ID, or sequence of variable IDs
-                          (one for each of the main ``vids``) to take values
-                          from. If provided, the generated columns will have
-                          values from the column(s) of this variable, instead
-                          of containinng binary 0/1 values. A ``take``
-                          variable must have columns that match the columns of
-                          the corresponding ``vid`` (by visits and instances).
-
-    :arg fillval:         If ``take`` is provided, the value to use for
-                          ``False`` rows. Defaults to ``np.nan``
-
-    :arg replaceTake:     If ``True`` (the default), and ``takeFrom`` variables
-                          were specified, the columns associated with the
-                          ``take`` variables are returned for removal.
+    If the ``minpres`` option is specified, it is used as a threshold -
+    categorical values with less than this many occurrences will not be added
+    as columns.
 
     The ``nameFormat`` argument controls how the new data columns should be
     named - it must be a format string using named replacement fields
@@ -412,6 +432,29 @@ def binariseCategorical(dtable,
     ``True``         ``False``           ``'{vid}-{value}.{instance}'``
     ``True``         ``True``            ``'{vid}-0.{value}'``
     ================ =================== ======================================
+
+    The ``replace`` option controls whether the original un-binarised columns
+    should be removed - it defaults to ``True``, which will cause them to be
+    removed.
+
+    By default, the new binary columns (one for each unique value in the
+    input columns) will contain a ``1`` indicating that the value is present,
+    or a ``0`` indicating its absence. As an alternative to this, the ``take``
+    option can be used to specify *another* variable from which to take values
+    when populating the output columns. ``take`` may be set to a variable ID,
+    or sequence of variable IDs (one for each of the input variables) to take
+    values from. If provided, the generated columns will have values from the
+    column(s) of this variable, instead of containinng binary 0/1 values.
+
+    A ``take`` variable must have columns that match the columns of the
+    corresponding variable (by both visits and instances).
+
+    If ``take`` is being used, the ``fillval`` option can be used to specify the
+    the value to use for ``False`` / ``0`` rows. It defaults to ``np.nan``.
+
+    The ``replaceTake`` option is similar to ``replace`` - it controls whether
+    the columns associated with the ``take`` variables are removed (``True`` -
+    the defailt), or retained.
     """  # noqa
 
     # get groups of columns for vid, grouped
@@ -527,22 +570,22 @@ def binariseCategorical(dtable,
 
 @custom.processor()
 def expandCompound(dtable, vids, nameFormat=None, replace=True):
-    """expandCompound([nameFormat], [replace])
+    """expandCompound([nameFormat][, replace])
     Expand a compound column into a set of columns, one for each value.
 
     Expands compound variables into a set of columns, one for each value.
     Rows with different number of values are padded with ``np.nan``.
 
-    :arg dtable:     The :class:`.DataTable`
+    This procedure is applied independently to each column of each specified
+    variable.
 
-    :arg vids:       Sequence of variable IDs to (independently) apply the
-                     expansion to.
+    The ``nameFormat`` option can be used to control how the new columns
+    should be named - it must be a format string using named replacement
+    fields ``'vid'``, ``'visit'``, ``'instance'``, and ``'index'``.  The
+    default value for ``nameFormat`` is ``'{vid}-{visit}.{instance}_{index}'``.
 
-    :arg nameFormat: Format string defining how the new columns should be named
-                     - see below.
-
-    :arg replace:    If ``True`` (the default), the original columns are
-                     flagged for removal.
+    The ``replace`` option controls whether the original columns are removed
+    (``True`` - the default), or retained.
     """
 
     if nameFormat is None:
