@@ -51,25 +51,7 @@ def isSparse(
 ) -> Tuple[bool, Union[str, None], Any]:
     """Returns ``True`` if the given data looks sparse, ``False`` otherwise.
 
-    Used by :func:`.removeIfSparse`.
-
-    The check is based on the following criteria:
-
-     - The number/proportion of non-NA values must be greater than
-       or equal to ``minpres``.
-
-     - The standard deviation of the data must be greater than ``minstd``.
-
-     - For integer and categorical types, the number/proportion of the largest
-       category must be less than ``maxcat``.
-
-     - For integer and categorical types, the number/proportion of the largest
-       category must be greater than ``mincat``.
-
-    If any of these criteria are not met, the data is considered to be sparse.
-
-    Each criteria can be disabled by passing in ``None`` for the relevant
-    parameter.
+    Used by :func:`.removeIfSparse` - see that function for details.
 
     By default, the ``minstd`` test is only performed on numeric columns, and
     the ``mincat``/``maxcat`` tests are only performed on integer/categorical
@@ -81,26 +63,6 @@ def isSparse(
     :arg ctype:    The series column type (one of the :attr:`.util.CTYPES`
                    values), or ``None`` to disable type-specific logic.
 
-    :arg minpres:  Minimum number/proportion of values which must be present.
-
-    :arg minstd:   Minimum standard deviation, for numeric/categorical types.
-
-    :arg mincat:   Minimum size/proportion of smallest category,
-                   for integer/categorical types.
-
-    :arg maxcat:   Maximum size/proportion of largest category,
-                   for integer/categorical types.
-
-    :arg abspres:  If ``True`` (the default), ``minpres`` is interpreted as
-                   an absolute count. Otherwise ``minpres`` is interpreted
-                   as a proportion.
-
-    :arg abscat:   If ``True`` (the default), ``mincat`` and ``maxcat`` are
-                   interpreted as absolute counts. Otherwise ``mincat`` and
-                   ``maxcat`` are interpreted as proportions
-
-    :arg naval:    Value to consider as "missing" - defaults to ``np.nan``.
-
     :returns:      A tuple containing:
 
                     - ``True`` if the data is sparse, ``False`` otherwise.
@@ -305,7 +267,8 @@ def pairwiseRedundantColumns(
 def matrixRedundantColumns(
         data      : pd.DataFrame,
         corrthres : float,
-        nathres   : Optional[float] = None) -> List[Tuple[int, int]]:
+        nathres   : float = None,
+        precision : str   = None) -> List[Tuple[int, int]]:
     """Identifies redundant columns based on their correlation with each
     other using dot products to calculate a correlation matrix.
 
@@ -319,6 +282,12 @@ def matrixRedundantColumns(
                     ``corrthres`` *and* a missing-value correlation greater
                     than ``nathres`` to be identified as redundant.
 
+    :arg precision: ``'double'`` (the default) or ``'single'``, specifying
+                    the floating point precision to use. Note that the
+                    algorithm used to calculate the correlation values is
+                    unstable for data with a range larger than ~10e5, so
+                    double precision is used as default.
+
     :returns:       Sequence of tuples of column indices, where each tuple
                     ``(a, b)`` indicates that column ``a`` is redundant with
                     respect to column ``b``.
@@ -327,11 +296,17 @@ def matrixRedundantColumns(
     if len(data.columns) < 2:
         return []
 
+    if precision == 'single':
+        dtype = np.float32
+    elif precision in (None, 'double'):
+        dtype = np.float64
+    else:
+        raise ValueError(f'Invalid precision: {precision}')
+
     # Keep a copy of the column names for logging.
-    # Create a 2D matrix containing all data, using
-    # float32 to limit memory consumption.
+    # Create a 2D matrix containing all data
     columns = data.columns
-    data    = data.to_numpy(dtype=np.float32, copy=True)
+    data    = data.to_numpy(dtype=dtype, copy=True)
 
     namask   = np.isnan(data)
     nacounts = namask.sum(axis=0)
@@ -347,7 +322,7 @@ def matrixRedundantColumns(
     data[namask] = 0
 
     # p=present elements
-    namask  = (~namask).astype(np.float32)
+    namask  = (~namask).astype(dtype)
     Ap = Bp = namask
     A  = B  = data
 
