@@ -217,7 +217,7 @@ def test_codeToNumeric():
     hier  = hierarchy.getHierarchyFilePath(name='opcs3')
     hier  = hierarchy.loadHierarchyFile(hier)
     codes = list(random.sample(hier.codings, 20)) + ['badcode']
-    exp   = [hier.index(c) + 1 for c in codes[:-1]] + [np.nan]
+    exp   = [hier.index(c) for c in codes[:-1]] + [np.nan]
 
     with tempdir():
         with open('data.txt', 'wt') as f:
@@ -714,7 +714,10 @@ def test_flattenHierarchical():
         with open('data.txt', 'wt') as f:
             f.write(data)
 
-        with open('hier.txt', 'wt') as f:
+        # the encoding number used here (2) is a real
+        # one which uses integer coding values. See
+        # hierarchy.codingDataType
+        with open('coding2.tsv', 'wt') as f:
             f.write(hier)
 
         mgr = mp.Manager()
@@ -731,7 +734,7 @@ def test_flattenHierarchical():
                                          mgr=mgr)
 
             with mock.patch('funpack.hierarchy.getHierarchyFilePath',
-                            return_value='hier.txt'):
+                            return_value='coding2.tsv'):
                 cfns.flattenHierarchical(dt, 1, level)
 
             vals = dt[:, '1-0.0'].values
@@ -741,3 +744,80 @@ def test_flattenHierarchical():
 
         dt = None
         mgr = None
+
+
+
+def test_flattenHierarchical_numeric():
+
+    data = tw.dedent("""
+    eid,1-0.0
+    1,10
+    2,20
+    3,30
+    4,40
+    5,50
+    6,60
+    7,70
+    8,80
+    9,90
+    10,
+    """).strip()
+
+    hier = tw.dedent("""
+    coding	meaning	node_id	parent_id
+    10	meaning 10	1	0
+    20	meaning 20	2	0
+    30	meaning 30	3	1
+    40	meaning 40	4	1
+    50	meaning 50	5	4
+    60	meaning 60	6	2
+    70	meaning 70	7	2
+    80	meaning 80	8	6
+    90	meaning 90	9	8
+    """).strip()
+
+    # level, expected
+    tests = [
+        (0, [1, 2, 1, 1, 1, 2, 2, 2, 2]),
+        (1, [1, 2, 3, 4, 4, 6, 7, 6, 6]),
+        (2, [1, 2, 3, 4, 5, 6, 7, 8, 8]),
+        (3, [1, 2, 3, 4, 5, 6, 7, 8, 9]),
+        (4, [1, 2, 3, 4, 5, 6, 7, 8, 9]),
+    ]
+
+    with tempdir():
+
+        with open('data.txt', 'wt') as f:
+            f.write(data)
+
+        # the encoding number used here (2) is a real
+        # one which uses integer coding values. See
+        # hierarchy.codingDataType
+        with open('coding2.tsv', 'wt') as f:
+            f.write(hier)
+
+        mgr = mp.Manager()
+        vartable, proctable, cattable, _ = gen_tables([1])
+
+        for level, exp in tests:
+
+            finfo = fileinfo.FileInfo('data.txt')
+            dt, _ = importing.importData(finfo,
+                                         vartable,
+                                         proctable,
+                                         cattable,
+                                         njobs=mp.cpu_count(),
+                                         mgr=mgr)
+
+            with mock.patch('funpack.hierarchy.getHierarchyFilePath',
+                            return_value='coding2.tsv'):
+                cfns.flattenHierarchical(dt, 1, level,
+                                         convertNumeric=True)
+
+            vals = dt[:, '1-0.0'].values
+
+            assert pd.isna(vals[ -1])
+            assert np.all( vals[:-1] == exp)
+
+        dt = None
+        mgr = None
