@@ -465,6 +465,7 @@ def test_main_subjects():
         mask[:7] = 1
         got  = pd.read_csv('out.tsv', delimiter='\t', index_col=0)
         exp  = dt[mask, :]
+
         assert np.all(got == exp)
 
 
@@ -1560,7 +1561,7 @@ def test_ids_only():
 
 
 @patch_logging
-def test_dupe_columns():
+def test_dupe_columns_rename():
     data       = np.random.randint(1, 100, (10, 3))
     data[:, 0] = np.arange(1, 10 + 1)
     colnames   = ['eid', '1-0.0', '1-0.0']
@@ -1577,7 +1578,7 @@ def test_dupe_columns():
 
 
 @patch_logging
-def test_dupe_columns_multiple_files():
+def test_dupe_columns_multiple_files_rename():
     data1       = np.random.randint(1, 100, (10, 3))
     data1[:, 0] = np.arange(1, 10 + 1)
     colnames1   = ['eid', '1-0.0', '2-0.0']
@@ -1599,3 +1600,107 @@ def test_dupe_columns_multiple_files():
             ['eid', '1-0.0', '2-0.0', '2-0.0.1', '3-0.0']
         exp = np.hstack((data1, data2[:, 1:]))
         assert np.all(df.to_numpy() == exp)
+
+
+@patch_logging
+def test_dupe_columns_remove():
+    data       = np.random.randint(1, 100, (10, 6))
+    data[:, 0] = np.arange(1, 10 + 1)
+    colnames   = ['eid', '1-0.0', '1-0.0', '2-0.0', '3-0.0', '3-0.0']
+
+    with tempdir():
+        with open('data.txt', 'wt') as f:
+            f.write('\t'.join(colnames) + '\n')
+            np.savetxt(f, data, fmt='%i', delimiter='\t')
+
+        main.main('-rm out.csv data.txt'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == ['eid',   '1-0.0', '2-0.0', '3-0.0']
+        assert np.all(df.to_numpy() == data[:, [0, 1, 3, 4]])
+
+
+@patch_logging
+def test_dupe_columns_multiple_files_remove():
+    data1       = np.random.randint(1, 100, (10, 5))
+    data1[:, 0] = np.arange(1, 10 + 1)
+    colnames1   = ['eid', '1-0.0', '1-0.0', '2-0.0', '3-0.0']
+    data2       = np.random.randint(1, 100, (10, 6))
+    data2[:, 0] = np.arange(1, 10 + 1)
+    colnames2   = ['eid', '2-0.0', '3-0.0', '4-0.0', '4-0.0', '5-0.0']
+
+    with tempdir():
+        with open('data1.txt', 'wt') as f:
+            f.write('\t'.join(colnames1) + '\n')
+            np.savetxt(f, data1, fmt='%i', delimiter='\t')
+        with open('data2.txt', 'wt') as f:
+            f.write('\t'.join(colnames2) + '\n')
+            np.savetxt(f, data2, fmt='%i', delimiter='\t')
+
+        main.main('-rm out.csv data1.txt data2.txt'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == \
+            ['eid', '1-0.0', '2-0.0', '3-0.0', '4-0.0', '5-0.0']
+        exp = np.hstack((data1[:, [0, 1, 3, 4]], data2[:, [3, 5]]))
+        assert np.all(df.to_numpy() == exp)
+
+
+@patch_logging
+def test_exclude_variables():
+    data       = np.random.randint(1, 100, (10, 11))
+    data[:, 0] = np.arange(1, 11)
+    colnames   = ['eid'] + [f'{v}-0.0' for v in range(1, 11)]
+
+    with tempdir():
+        with open('data.tsv', 'wt') as f:
+            f.write('\t'.join(colnames) + '\n')
+            np.savetxt(f, data, fmt='%i', delimiter='\t')
+
+        main.main('-ev 3,5,8 out.csv data.tsv'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == \
+            ['eid', '1-0.0', '2-0.0', '4-0.0', '6-0.0', '7-0.0', '9-0.0', '10-0.0']
+        exp = data[:, [0, 1, 2, 4, 6, 7, 9, 10]]
+        assert np.all(df.to_numpy() == exp)
+
+        main.main('-ow -v 1:9 -ev 3:5 out.csv data.tsv'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == \
+            ['eid', '1-0.0', '2-0.0','6-0.0', '7-0.0', '8-0.0', '9-0.0']
+        exp = data[:, [0, 1, 2, 6, 7, 8, 9]]
+        assert np.all(df.to_numpy() == exp)
+
+
+
+@patch_logging
+def test_exclude_categories():
+    data       = np.random.randint(1, 100, (10, 11))
+    data[:, 0] = np.arange(1, 11)
+    colnames   = ['eid'] + [f'{v}-0.0' for v in range(1, 11)]
+
+    with tempdir():
+
+        with open('cats.tsv', 'wt') as f:
+            f.write(tw.dedent("""
+            ID\tCategory\tVariables
+            1\tcat1\t1:4
+            2\tcat2\t4:8
+            3\texclude\t3:6
+            """.strip()))
+
+        with open('data.tsv', 'wt') as f:
+            f.write('\t'.join(colnames) + '\n')
+            np.savetxt(f, data, fmt='%i', delimiter='\t')
+
+        main.main('-cf cats.tsv -ec exclude out.csv data.tsv'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == \
+            ['eid', '1-0.0', '2-0.0', '7-0.0', '8-0.0', '9-0.0', '10-0.0']
+        exp = data[:, [0, 1, 2, 7, 8, 9, 10]]
+        assert np.all(df.to_numpy() == exp)
+
+        main.main('-cf cats.tsv -ow -c cat1 -c cat2 -ec exclude out.csv data.tsv'.split())
+        df = pd.read_csv('out.csv')
+        assert list(df.columns) == \
+            ['eid', '1-0.0', '2-0.0', '7-0.0', '8-0.0']
+        exp = data[:, [0, 1, 2, 7, 8]]
+        assert np.all(df.to_numpy() == exp)
